package models

import (
	"errors"
	"fmt"
	"reflect"
	"strings"
	"time"

	"github.com/astaxie/beego/orm"
)

type CompanyUser struct {
	Id         int       `orm:"column(id);"`
	Name       string    `orm:"column(name)"`
	Role       int16     `orm:"column(role)"`
	Profile    string    `orm:"column(profile)"`
	ModifiedBy int64     `orm:"column(modifiedBy);"`
	Created    time.Time `orm:"column(created);type(timestamp with time zone);auto_now_add"`
	Modified   time.Time `orm:"column(modified);type(timestamp with time zone);auto_now_add"`
}

func (t *CompanyUser) TableName() string {
	return "company_user"
}

func init() {
	orm.RegisterModel(new(CompanyUser))
}

// AddCompanyUser insert a new CompanyUser into database and returns
// last inserted Id on success.
func AddCompanyUser(o orm.Ormer, m *CompanyUser) (id int64, err error) {
	id, err = o.Insert(m)
	return
}

// GetCompanyUserById retrieves CompanyUser by Id. Returns error if
// Id doesn't exist
func GetCompanyUserById(o orm.Ormer, id int) (v *CompanyUser, err error) {
	v = &CompanyUser{Id: id}
	if err = o.Read(v); err == nil {
		return v, nil
	}
	return nil, err
}

// GetAllCompanyUser retrieves all CompanyUser matches certain condition. Returns empty list if
// no records exist
func GetAllCompanyUser(o orm.Ormer, query map[string]string, fields []string, sortby []string, order []string,
	offset int64, limit int64) (ml []interface{}, err error) {
	qs := o.QueryTable(new(CompanyUser))
	// query k=v
	for k, v := range query {
		// rewrite dot-notation to Object__Attribute
		k = strings.Replace(k, ".", "__", -1)
		if strings.Contains(k, "isnull") {
			qs = qs.Filter(k, (v == "true" || v == "1"))
		} else {
			qs = qs.Filter(k, v)
		}
	}
	// order by:
	var sortFields []string
	if len(sortby) != 0 {
		if len(sortby) == len(order) {
			// 1) for each sort field, there is an associated order
			for i, v := range sortby {
				orderby := ""
				if order[i] == "desc" {
					orderby = "-" + v
				} else if order[i] == "asc" {
					orderby = v
				} else {
					return nil, errors.New("Error: Invalid order. Must be either [asc|desc]")
				}
				sortFields = append(sortFields, orderby)
			}
			qs = qs.OrderBy(sortFields...)
		} else if len(sortby) != len(order) && len(order) == 1 {
			// 2) there is exactly one order, all the sorted fields will be sorted by this order
			for _, v := range sortby {
				orderby := ""
				if order[0] == "desc" {
					orderby = "-" + v
				} else if order[0] == "asc" {
					orderby = v
				} else {
					return nil, errors.New("Error: Invalid order. Must be either [asc|desc]")
				}
				sortFields = append(sortFields, orderby)
			}
		} else if len(sortby) != len(order) && len(order) != 1 {
			return nil, errors.New("Error: 'sortby', 'order' sizes mismatch or 'order' size is not 1")
		}
	} else {
		if len(order) != 0 {
			return nil, errors.New("Error: unused 'order' fields")
		}
	}

	var l []CompanyUser
	qs = qs.OrderBy(sortFields...)
	if _, err = qs.Limit(limit, offset).All(&l, fields...); err == nil {
		if len(fields) == 0 {
			for _, v := range l {
				ml = append(ml, v)
			}
		} else {
			// trim unused fields
			for _, v := range l {
				m := make(map[string]interface{})
				val := reflect.ValueOf(v)
				for _, fname := range fields {
					m[fname] = val.FieldByName(fname).Interface()
				}
				ml = append(ml, m)
			}
		}
		return ml, nil
	}
	return nil, err
}

// UpdateCompanyUserById updates CompanyUser by Id and returns error if the record to be updated doesn't exist
func UpdateCompanyUserById(o orm.Ormer, m *CompanyUser) (err error) {
	m.Modified = time.Now()
	v := CompanyUser{Id: m.Id}
	// ascertain id exists in the database
	if err = o.Read(&v); err == nil {
		var num int64
		m.Created = v.Created
		if num, err = o.Update(m); err == nil {
			fmt.Println("Number of records updated in database:", num)
		}
	}
	return
}

// DeleteCompanyUser deletes CompanyUser by Id and returns error if
// the record to be deleted doesn't exist
func DeleteCompanyUser(o orm.Ormer, id int) (err error) {
	v := CompanyUser{Id: id}
	// ascertain id exists in the database
	if err = o.Read(&v); err == nil {
		var num int64
		if num, err = o.Delete(&CompanyUser{Id: id}); err == nil {
			fmt.Println("Number of records deleted in database:", num)
		}
	}
	return
}
